/** 
 * Button 
 *
 * This class creates a button object with center position (xpos, ypos), width 
 * xsize, height ysize, all in pixels, and color c.
 *
 * @author Sarah Greer
 * @see TextBox
 * @see RadioButton
 * @see SmallButton
 *
 */


public class Button {
  color c, c2, c3;      // colors for fill, stroke, and text
  float xpos, ypos;     // center x and y of button
  float xsize, ysize;   // width and height of button
  float x2, y2;         // half width and height of button (for borders)
  int sw;               // stroke weight
  String txt = "";      // button text
  float txtsize;        // text size
  color hoverc = color(0,0,0,50);       // shaded darker when hovered
  color clickc = color(0,0,0,125);      // shaded darker when clicked
  
  public Button(float xpos, float ypos, float xsize, float ysize, color c) {
    this.xpos = xpos;
    this.ypos = ypos;
    this.xsize = xsize;
    this.ysize = ysize;
    this.c = c;
    x2 = xsize/2;
    y2 = ysize/2;

    // default stroke color and weight 
    c2 = #000000;
    sw = 3;
  }

  // set fill color
  protected void setF(color c){
    this.c = c;
  }

  // set stroke color
  protected void setS(color c){
    this.c2 = c;
  }

  // set stroke weight
  protected void setStrokeWeight(int sw){
    this.sw = sw;
  }

  // set button text
  protected void setText(int txtsize, String txt, color c){
    this.txtsize = txtsize;
    textSize(txtsize);
    this.txt = txt;
    this.c3 = c;
  }
  
  public void display() {
    stroke(c2);
    strokeWeight(sw);

    fill(c);
    rectMode(CENTER);
    rect(xpos,ypos,xsize,ysize);

    // fill with clicked color
    if ( isClicked() ){ 
      fill(clickc);
      rect(xpos,ypos,xsize,ysize);
    
    // fill with hover color
    } else if ( isHovering() ){
      fill(hoverc);
      rect(xpos,ypos,xsize,ysize);
    }

    // add text if there is any
    if (txt != ""){
      textSize(txtsize);
      fill (c3);
      textAlign(CENTER, CENTER);
      text(txt, xpos, ypos);
    }
  }
  
  // checks if button is being hovered over
  protected boolean isHovering(){
    if (mouseX > xpos-x2 && mouseX < xpos+x2 &&
        mouseY > ypos-y2 && mouseY < ypos+y2) return true; 
    else return false; 
  }

  // checks if button is clicked
  protected boolean isClicked() {
    if (isHovering() && mousePressed) return true; 
    else return false; 
  }
}
