/**
 * Function
 *
 * This class displays a (mathematical) function on a plot object from the 
 * (computer science) function called "function"... fewf.
 *
 * @author Sarah Greer
 * @see Plot
 * @see Line
 * @see VL
 *
 */
class Function{
  Plot plot;            // plot to display function on
  float[] x, f;         // array to store  x and y in original coordinates
  float[] xmap, fmap;   // array to store x and y in pixel coordinates
  int nx;               // number of samples in x
  float miny, minx, maxx;   // min and max x and y coordinates
  float dy, dx, dxdy;   // conversion factors for x and y to xmap and ymap
  color c;              // color of function
  boolean derline = true;   // show derivative line?

  Function(Plot plot){
    this.plot = plot;
    graphUpdate();

    // function values
    f = new float[nx];
    fmap = new float[nx];

    c = #3CB371;
    generateFunction();
  }

  float getFval(int val){ return f[val]; }
  
  // udate graph
  void graphUpdate(){
    x = plot.x;
    xmap = plot.xmap;
    nx = plot.nx;
    miny = plot.miny;
    minx = plot.minx;
    maxx = plot.maxx;
    dy = plot.dy;
    dx = plot.dx;
    dxdy = dx/dy;
  }

  // first derivative
  float derivative(int xval){
    // central differencing
    float der = 1; 
    if (xval > 0 && xval < f.length-1){
      der = (f[xval-1] - f[xval+1])/(2*dx); }
    return der;
  }

  // map of derivative on pixels
  float derivativeMap(int xval){
    return derivative(xval)*dxdy;
  }

  // second derivative 
  float derivative2(int xval){
    // central differencing
    float der = 1; 
    if (xval > 0 && xval < f.length-1){
      der = (f[xval+1] -2*f[xval] + f[xval-1])/(dx*dx); }
    return der;
  }

  // map of second derivative on pixels
  float derivative2Map(int xval){
    return derivative2(xval)*dxdy;
  }

  // change color of function
  void changeColor(color c){
    this.c = c;
  }

  // get the function values
  void generateFunction(){
    for (int i = 0; i < nx; i++){
      f[i] = -1*functionForDisplay(x[i]);
      fmap[i] = (f[i] - miny)/dy;
    }
  }

  // derivative line to follow cursor
  void derLine(){
    fill(#CC3232);
    stroke(#CC3232);
    textAlign(LEFT);

    // check if mouse is on plot
    if (mouseX > 0 && mouseX < x.length){
      line(mouseX,0,mouseX,height);
      ellipse(mouseX,fmap[mouseX],10,10);
      if(derline){
        float der = derivativeMap(mouseX);
        float s = 50;
        float s2 = 25;
        strokeWeight(2);
        line(mouseX-s, fmap[mouseX] + der*s, mouseX+s, fmap[mouseX]-der*s);
        stroke(0,0,255,50);

        // plot second derivative
        //line(mouseX-s2, fmap[mouseX] + der2*s2, mouseX+s2, fmap[mouseX]-der2*s2);
        //float der2 = derivative2Map(mouseX);
        stroke(#ff0000);
      }
    }
  }

  // display x and y coordinates of mouse
  void coords(){
    if (mouseX > 0 && mouseX < x.length){
      fill(#696969);
      textSize(20);
      text("("+x[mouseX]+","+ -1*f[mouseX]+")", 20,40);
      text("dy/dx = " + derivative(mouseX), 20,65);

      // display second derivative...
      //text("d2y/dx2 = " + derivative2(mouseX), 20,90);
    }

  }

  // display function
  void display(){
    fill(c);
    stroke(c);
    strokeWeight(4);
    for (int i = 1; i < nx; i++){
      line(xmap[i-1],fmap[i-1],xmap[i],fmap[i]);
    }
  }
}
