/**
 *
 * Line
 *
 * This class defines a line object. In Newton's method, it is the slope at
 * the (x, y) pair of this iteration
 *
 * @author Sarah Greer
 * @see VL
 * @see Plot
 * @see Function
 *
 */

public class Line {
  Function funct;           // function to use
  Plot plot;                // plot to display on 
  float m;                  // slope
  float[] x, f;             // array to store x and y in original coordinates
  float[] xmap, fmap;       // array to store x and y in pixel coordinates
  float x1, y1;             // initial x and y
  float b;                  // y0
  int nx;                   // number of samples in x
  float minx, miny, maxx, maxy; // min and max x and y coordinates
  float dy, dx, dxdy;       // conversion factors for x and y to xmap and ymap
  float mx;                 // index in array
  color c;                  // color of line
  

  public Line(Plot plot, Function funct, int mx){
    this.plot = plot;
    this.funct = funct;
    this.mx = mx;
    x = funct.x;
    
    // get initial x, y, and slope values
    x1 = x[mx];
    y1 = -1* funct.getFval(mx);
    m = funct.derivative(mx);

    // find y0
    b = y1 - m * x1;

    // get function and map
    nx = plot.nx;
    fmap = new float[nx];
    f = new float[nx];

    // update graph
    graphUpdate();
    generateFunction();
    c = #4c4cff;
  }

  float lineFunct(float x){
    return m*x+b;
    //return m*(x - x1) + y1;
  }

  // function to update graph
  void graphUpdate(){

    // get necessary values for display from plot
    x = plot.x;
    xmap = plot.xmap;
    nx = plot.nx;
    miny = plot.miny;
    minx = plot.minx;
    maxx = plot.maxx;
    dy = plot.dy;
    dx = plot.dx;
    dxdy = dx/dy;
    generateFunction();

    // find x, y, and slope
    x1 = x[(int)mx];
    y1 = -1* funct.getFval((int)mx);
    m = funct.derivative((int)mx);
  }

  // generate the function for display
  void generateFunction(){
    for (int i = 0; i < nx; i++){
      f[i] = -1*lineFunct(x[i]);
      fmap[i] = (f[i] - miny)/dy;
    }
  }

  // map of slope
  float getBMap(){
    return ( - miny)/dy;
  }

  // map of x0
  int getXMap0(){
    return int((-b/m-minx)/dx);
  }

  // set color of line
  void setColor(color c){
    this.c = c;
  }

  // display line
  void display(){
    for (int i = 1; i < nx; i++){
      fill(c);
      stroke(c);
      strokeWeight(2);

      // above x axis
      if (y1 >= 0 && fmap[i] < getBMap()){
        if ( (m > 0 && i < mx ) || (m < 0 && i > mx) ){
          line(xmap[i-1],fmap[i-1],xmap[i],fmap[i]);
        }

      // below x axis
      } else if (y1 < 0  && fmap[i] > getBMap()){
        if ( (m < 0 && i < mx ) || (m > 0 && i > mx) ){
          line(xmap[i-1],fmap[i-1],xmap[i],fmap[i]);
        }
      }
    }
  }
}
