/**
 * RadioButton
 *
 * This class creates a selection area of radio buttons
 *
 * @author Sarah Greer
 * @see Button
 * @see SmallButton
 *
 */

class RadioButton { 
  float xpos, ypos, r;  // (x, y) start coordinates and size of button
  color c;              // color of button
  float offset;         // offset between buttons
  int num;              // number of buttons
  float txtsize;        // size of text
  SmallButton[] buttons;    // array to hold all buttons
  String[] txt;             // array to hold button text
  Boolean[] selectedVals;   // array to hold selected value


  RadioButton(float xpos, float ypos, float r, color c, float offset, String[] txt){
    this.xpos = xpos;
    this.ypos = ypos;
    this.r = r;
    this.c = c;
    this.offset = offset;
    num = txt.length;

    // set buttons and text
    buttons = new SmallButton[num];
    this.txt = new String[num];
    selectedVals = new Boolean[num];
    for (int i=0; i<num; i++){
      buttons[i] = new SmallButton(xpos, ypos+i*offset, r, c, true);
      this.txt[i] = txt[i];
      selectedVals[i] = false;
    }
    selectedVals[0] = true;
    txtsize = 15;

  }

  // display buttons
  void display(){
    select();
    checkClick();
    textSize(txtsize);
    textAlign(LEFT,CENTER);
    for (int i=0; i<num; i++){
      buttons[i].display();
      fill(#000000);
      text(txt[i], xpos + r, ypos+i*offset-0.1*r); 
    }
  }

  // check if the button has been clicked
  void checkClick(){
    for (int i=0; i<num; i++){
      if (selectedVals[i] == true){
        buttons[i].select();
      } else {
        buttons[i].unselect();
      }
    }
  }

  // select a button
  void select(){
    for (int i=0; i<num; i++){
      if (buttons[i].isClicked() == true){
        setTrue(i);
      } 
    }
  }

  // make a certain button true and others false
  void setTrue(int index){
    for (int i=0; i<num; i++){
      selectedVals[i] = false;
    }
    selectedVals[index] = true;
    checkClick();
  }

  // find which button selected
  int selectedButton(){
    int sele = 2;
    for (int i=0; i<num; i++){
      if(selectedVals[i] == true){ 
        sele = i;
      }
    }
    return sele;
  }

}
