/**
 * SmallButton
 *
 * This class acts as a helper class for radio buttons and check boxes. 
 * It represents a single button inside a list of buttons
 * 
 * @author Sarah Greer
 * @see Button
 * @see RadioButton
 * @see CheckBox
 *
 */

class SmallButton extends Button{
  float r;              // size of button
  boolean circ;         // circle (radio) or square (check box)
  boolean isSelected;   // button is selected

  SmallButton(float xpos, float ypos, float r, color c, boolean circ){
    super(xpos, ypos, r, r, c);
    this.r = r;
    sw = 1;
    this.circ = circ;
    isSelected = true;
  }

  // select button
  void select(){
    isSelected = true;
  }

  // unselect button
  void unselect(){
    isSelected = false;
  }

  // toggle between selected and unselected
  void toggle(){
    isSelected = !isSelected;
  }

  // display button
  void display(){
    stroke(c2);
    strokeWeight(sw);
    fill(c);

    // circle button
    if (circ){ 
      ellipse(xpos,ypos,r,r); 
      if ( isClicked() ){ 
        fill(clickc);
        ellipse(xpos,ypos,r,r);
      } else if ( isHovering() ){
        fill(hoverc);
        ellipse(xpos,ypos,r,r);
      }
      if (isSelected){
        fill(0);
        ellipse(xpos,ypos,r-0.5*r,r-0.5*r);
      }

    // square button
    } else { 
      rect(xpos,ypos,r,r); 
      if ( isClicked() ){ 
        fill(clickc);
        rect(xpos,ypos,r,r);
      } else if ( isHovering() ){
        fill(hoverc);
        rect(xpos,ypos,r,r);
      }
      if (isSelected){
        fill(0);
        rect(xpos,ypos,r-0.5*r,r-0.5*r);
      }
    }

    // set text
    if (txt != ""){
      fill (c3);
      textAlign(CENTER, CENTER);
      text(txt, xpos, ypos);
    }
  }

}
